<?php

declare(strict_types=1);

namespace Doctrine\ORM\Mapping\Driver;

use Doctrine\Persistence\Mapping\Driver\MappingDriverChain;

/**
 * {@inheritDoc}
 *
 * @deprecated this driver will be removed. Use Doctrine\Persistence\Mapping\Driver\MappingDriverChain instead
 */
class DriverChain extends MappingDriverChain
{
}
