<?php

declare(strict_types=1);

namespace Doctrine\ORM\Proxy;

use Doctrine\Common\Proxy\Proxy as BaseProxy;

/**
 * Interface for proxy classes.
 *
 * @deprecated 2.14. Use \Doctrine\Persistence\Proxy instead
 */
interface Proxy extends BaseProxy
{
}
