<?php

declare(strict_types=1);

$metadata->mapField(
    [
        'id' => true,
        'fieldName' => 'id',
        'type' => 'integer',
    ]
);
$metadata->mapField(
    [
        'fieldName' => 'count',
        'type' => 'integer',
        'columnName' => '`count`',
    ]
);
